/*  
 *  bpassdel.c
 *
 *  Version 1.3
 *  26 July 1999 
 *
 *   DELETE USER AND PASSWORD FROM A PASSWORD DATABASE
 *
 *   Realised by MARTINEZ Sylvain  
 *
 *   Based on the BUGS crypt's algorithm of MARTINEZ Sylvain  
 *   (Big and Usefull Great Security)  
 *
 *   UNIX Administrator (Merrill Lynch, UK) 
 *   Network Security Consultant (ASI, France)
 *   Passion: Cryptology and Network Security  
 *   email: martinez@asi.fr  
 *          sylvain.martinez@netcourrier.com 
 *   URL: http://www.asi.fr/~martinez
 *
 *  Copyright (C) 1996,1997,1998,1999 MARTINEZ Sylvain  
 *  THIS IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER  
 *  THE TERMS OF THE GNU GENERAL PUBLIC LICENSE, see the file COPYING.  
 */

/*
 * HISTORY
 *
 * --- Version in progress ---
 *
 * --- V 1.3 ---
 *
 * 26/07/1999 : - Replaced sizeof() by strlen() while calling malloc()
 *  
 * 23/07/1999 : - Changed the way I handle parameters
 *
 * 09/07/1999 : - Changed pass_clear from char to unsigned char
 *
 *  --- V 1.2 ---
 *
 * 16/02/1998 : - Corrected the input password bug
 *		- Changed bcrypt initialisation (binit)
 *
 * --- V 1.1 ---
 *
 * 07/02/1998 : - Added the libcrypt version number
 *
 * --- V 1.0 ---
 *
 * 27/01/1998 : - First tests.
 *            
 */

/*
 * This header is important
 * There is some global variables that you will need, eg:
 * USER_LENGTH = size of the string that contain the user name
 * other reserved variable name : KEYLENGTH, varinit->NB_CHAR, varinit->NB_INDEX, etc
 * Read the documentation or have a look of this header for more information
 */
#include "../include/libcrypt.h"
#include "../include/extra.h"

char *VERSION="v1.3, July 1999";

/*
 * ASCII code for the RETURN charactere that may change on different OS 
 */
const BCRYPT_RETURN = 10;	

/*
   * Default : mode = 0 (non verbose)
   *           Key's length = 128 bits
   */
  int MODE = 0;
  int PARAM_KEY = 128;
  char *PARAM_USER = "";
  char *PARAM_FILE = "";
  globalvar *varinit;

  
/*
 * Functions
 */
void usage(void);
int argcheck(int, char **);
int init(void);
int delete(char **);

int 
main (int argc, char **argv)
{

  if (argcheck(argc, argv) == 0) return 0;
  if (init() == 0) return 0;
  if (delete(argv) == 0) return 0;

  return 1;
}
 
/*
 * Passwd generator function
 */
int delete(char **argv)
{
  char carac;
  int i, test, length;
  unsigned char *pass_clear;
  TYPE_INT *code_file;

  test = 1;

code_file = (TYPE_INT *) malloc(varinit->NB_CHAR);
pass_clear = (unsigned char *) malloc (varinit->NB_CHAR);

printf("\n BPASSDEL %s, Martinez Sylvain",VERSION);
printf("\n DELETE PASSWORD"); 
printf("\n Libcrypt version : '%s'", varinit->LIB_VERSION);
printf("\n\n Delete Password in progress.");
printf("\n KEY'S LENGTH used : %d.",PARAM_KEY);
printf("\n Password file : '%s'.\n",PARAM_FILE);

  if (bcrypt_read_passwd ("root", PARAM_FILE, code_file, MODE, varinit) == 0)
    {
      printf ("\n Only the user 'root' can delete a passwd.");
      printf("\n This user MUST exist in the passwd file.\n\n");
      return 0;
    }


      printf ("\n Administrator identification : 'root' ");

      printf ("\n Root password -> ");
      i = 0;
      do
	{
	  carac = bcrypt_vol (1);
	  if (carac != BCRYPT_RETURN) pass_clear[i] = carac;
          i++;
	}
      while (i < varinit->NB_CHAR && (carac != BCRYPT_RETURN));
      if (carac == BCRYPT_RETURN)
        {
        pass_clear[i - 1] = '\0';
        length = i - 1;
        }
       else
         length = i;

      if (blogin (code_file, pass_clear, length, 0, MODE, varinit) == 0)
          {
	   printf("\n Identification failed. \n\n");
	   return 0;
          }
  /* 
   * Deleting passwd 
   */
      printf ("\n Are you sure to DELETE user '%s' [y/n] ", PARAM_USER);
      carac = bcrypt_vol(1);

      if (carac != 'y') 
         {
	  printf("\n Delete passwd canceled.\n\n");
	  return 0;
         }

     if(bcrypt_delete_passwd(PARAM_FILE, PARAM_USER, PARAM_KEY, MODE, varinit) == 0)
	{
	 printf("\n ERROR.");
	 printf("\n user '%s' may not exist.",PARAM_USER);
	 printf("\n Delete passwd failed.\n\n");
         return 0;
        }

  printf("\n user '%s' deleted.\n\n",PARAM_USER);
  return 1;
}


/*
 * Initialised the global variables
 */
int init()
{
varinit = (globalvar *) malloc(sizeof(globalvar));
/*
 * The log will be generated in the stderr output
 */
binit(PARAM_KEY, 0, varinit);
return 1;
}

/* 
 * Check the parameter of the program
 */
int argcheck(int argc, char **argv)
{
 int i;

 for (i = 0; i < argc; i++)
     {
      if ((0 == strcmp(argv[i],"-f")) && (i+1 < argc))
	 {
	  PARAM_FILE = (char *) malloc(strlen(argv[i+1]));
	  strcpy(PARAM_FILE,argv[i+1]);
	 }
      else if ((0 == strcmp(argv[i],"-u")) && (i+1 < argc))
		{
		 PARAM_USER = (char *) malloc(strlen(argv[i+1]));
		 strcpy(PARAM_USER,argv[i+1]);
		}
      else if ((0 == strcmp(argv[i],"-k")) && (i+1 < argc))
		{
		 PARAM_KEY = atoi(argv[i+1]);
		}
      else if (0 == strcmp(argv[i],"-v")) MODE = 1;
     }

if (0 == strcmp(PARAM_USER, ""))
   {
    usage();
    return 0;
   }

if (fopen(PARAM_FILE,"rb") == NULL) 
   {
    printf("\n ERROR.");
    printf("\n The file '%s' does not seem to exist.\n\n",PARAM_FILE);
    return 0;
   }
	
return 1;
}

void usage()
{
   printf("\n BPASSDEL %s, (C) Martinez Sylvain",VERSION);
   printf("\n DELETE PASSWORD AND USER"); 
   printf("\n\n Usage: bpass -f passwd_file -u user_name {OPTIONS}");
   printf("\n\n {OPTIONS} : ");
   printf("\n\n -k nb :  KEYLENGTH which has to be a 2 multiple integer.");
   printf("\n	      At least 32.");
   printf("\n	      DEFAULT = 128"); 	
   printf("\n 	      Choose the same keylength that the one used to do the passwd");
   printf("\n\n  -v : Verbose mode.\n\n"); 
}
