/*  
 *  libcrypt.h
 *
 *
 *  B U G S     LIBRARY     HEADER  
 *
 *   CRYPTOGRAPHY ALGORITHM
 *   Version 2.0.0
 *   26 July 1999
 *
 *  -> make multi-users programms
 *  -> generate passwd
 *  -> crypt file
 *
 *   Realised by MARTINEZ Sylvain
 *
 *   Based on the BUGS crypt's algorithm of MARTINEZ Sylvain
 *   (Big and Usefull Great Security)
 *
 *   UNIX Administrator (Merrill Lynch, UK)
 *   Network Security Consultant (ASI, France)
 *   Passion: Cryptology and Network Security
 *   email: martinez@asi.fr   
 *          sylvain.martinez@netcourrier.com
 *   URL: http://www.asi.fr/~martinez
 *
 *  Copyright 1996,1997,1998,1999 MARTINEZ Sylvain
 *  THIS IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY IT UNDER
 *  THE TERMS OF THE GNU GENERAL PUBLIC LICENSE, see the file COPYING.
 */                                                                    

#ifndef _LIBCRYPT_H
#define _LIBCRYPT_H

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>

/*
 * Compatibility with Windows 95 Borland C++ DLL
 * With BC 4.5 or laster, replace the line :
 * #define RETURN_TYPE extern "C" __declspec(dllexport) int FAR PASCAL
 * by
 * #define RETURN_TYPE extern int FAR PASCAL 
 */
#if defined(__WIN32__)
#include <windows.h>
#define RETURN_TYPE extern __declspec(dllexport) int FAR PASCAL
#else 
#define RETURN_TYPE int 
#endif

/*
 * Type of int that I will use
 * Default = long = 4 bytes = 32 bits 
 * int = 2 bytes = 16 bits 
 * long long = 8 bytes = 64 bits
 *
 * Yes, I prefer to do a #define for that,
 * because if a system has no long type I can easly
 * change the type without changing my C source code. 
 */ 
#define TYPE_INT unsigned long

/*
 * Size of the string that contain the name of the user

#define USER_LENGTH 20
 */
/*
 * NB_BYTE   : Lenght in bits of the int that I use 
 * KEYLENGTH : Length of the key used to crypt
 * NB_INDEX  : Number of index of the aray that will contain the cipher text
 * NB_CHAR   : Number of caracter of the cipher text and of the clear text
 * NB_CHAR_DOUBLE,
 * NB_INDEX_DOUBLE : I only use these variable during the swap function 
 * VERSION   : Libcrypt version number (as a string)
 */

typedef struct
         {
          int NB_BYTE;
	  int KEYLENGTH;
	  int NB_INDEX;
	  int NB_CHAR;
	  int NB_INDEX_DOUBLE;
	  int NB_CHAR_DOUBLE;
	  int USER_LENGTH;
	  char LIB_VERSION[10]; 		
         }
globalvar;

/*
 * Log file
 */
FILE *BCRYPTLOG; 

/*
 * Crypt's function
 */

int bcrypt_transcription (unsigned char *, TYPE_INT *, TYPE_INT *,
                          TYPE_INT *, int, globalvar *);
int bcrypt_add (TYPE_INT *, TYPE_INT *,int, globalvar *);
int bcrypt_test_length (unsigned char *, int, int, globalvar *);
int bcrypt_swap (TYPE_INT *, TYPE_INT *, TYPE_INT *, int, globalvar *);
int bcrypt_code (int, TYPE_INT, TYPE_INT *, unsigned char *, int,
							       globalvar *);
TYPE_INT  bcrypt_read_key (TYPE_INT *, TYPE_INT *, int, globalvar *);
int bcrypt_comparison (TYPE_INT *, TYPE_INT *, int, globalvar *);
int bcrypt_test_passwd (int, TYPE_INT *, unsigned char *, int, int,
                        globalvar *);
int bcrypt_write_file (FILE *, FILE *, int *, int, TYPE_INT *,
                       TYPE_INT *, TYPE_INT *, int, globalvar *);
int bcrypt_write_file_prob (int, FILE *, FILE *, int *, int, TYPE_INT *,
                       unsigned char *, TYPE_INT *,TYPE_INT *, TYPE_INT *, int,
								globalvar *);
int bcrypt_read_keyfile (unsigned char *, char *, int, globalvar *);
int bcrypt_write_keyfile (unsigned char *, char *, int, globalvar *);

/*
 * Functions that maybe used in your program
 */
RETURN_TYPE binit  (int, int, globalvar *);
RETURN_TYPE bkey_generator (unsigned char *, int, char *,int, int, int,
								 globalvar *);
RETURN_TYPE bcrypt_write_hide (int, char *, char *, int);
RETURN_TYPE bcrypt_read_hide (int, char *, char *, int);
RETURN_TYPE bfile (int, char *, char *, char *, unsigned char *, int, int, int,
				 				 globalvar *);
RETURN_TYPE bpow(int , int);

/*
 * These functions are not used on Windows95
 */
int bpass (TYPE_INT *, unsigned char *, int, int, int, globalvar *);
int blogin (TYPE_INT *, unsigned char *, int, int, int, globalvar *);
int bcrypt_read_passwd (char *, char *, TYPE_INT *, int, globalvar *);
int bcrypt_write_passwd (char *, TYPE_INT *, char *, int, globalvar *);
int bcrypt_delete_passwd (char *, char *, int, int, globalvar *);

#endif
